# -*- coding: utf-8 -*-
"""
Created on Thu Jan 26 16:06:39 2023

@author: brack28
"""

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import pdb
from matplotlib.patches import Patch
from matplotlib.lines import Line2D
from matplotlib.text import OffsetFrom
from matplotlib.figure import figaspect
import seaborn as sns
from uncertainties import ufloat
from matplotlib import colors as clr
import matplotlib.transforms as mtransforms
from matplotlib.gridspec import GridSpec

## HZDR Colormap
HHBlue = '#005aa0ff'
HHB80 = '#2473adff'
HHB60 = '#6691c4ff'
HHB40 = '#9bb3d7ff'
HHB20 = '#cdd8ecff'
HHOrange = '#f0781eff'
HHO80 = '#f39150ff'
HHO60 = '#f7af7cff'
HHO40 = '#fbcba8ff'
HHO20 = '#fde6d4ff'
HHGreen = '#8cb423ff'
HHG80 = '#a1c445ff'
HHG60 = '#b8d46cff'
HHG40 = '#cce89eff'
HHG20 = '#e6f5ceff'
HHGrayDark = '#515151ff'
HHGrayMiddle = '#9c9c9cff'
HHGrayLight = '#b9b9b9ff'
# HZDR_Pal = [HHBlue, HHB80, HHB60, HHB40, HHB20, HHOrange, HHO80, HHO60, HHO40, HHO20, HHGreen, HHG80, HHG60, HHG40, HHG20, HHGrayDark, HHGrayMiddle, HHGrayLight]
HZDR_Pal = [HHBlue, HHB80, HHB60, HHB40, HHB20, HHO20, HHO40, HHO60, HHO80, HHOrange]
HZDR_palette = sns.color_palette(HZDR_Pal, as_cmap=True)
pal = clr.LinearSegmentedColormap.from_list(HZDR_palette, HZDR_Pal, N=256)
# sns.palplot(HZDR_palette)
# sns.palplot(pal)

myPal = {"ELBE_cw": 'palevioletred', "ELBE_p-like": HHB60, "ELBE_e-like": 'crimson', "ELBE_FLASH": HHBlue, 
         "UPTD_cw_Eppi": HHO60, "UPTD_FLASH_Eppi": HHOrange, "UPTD_cw_slab": HHO60, "UPTD_FLASH_slab": HHOrange,
         "Draco": HHGreen, "UPTD_cw":HHO60, "UPTD_FLASH": HHOrange, "empty": 'white'
}

## Daten einlesen
df_ZFE_length = pd.read_excel('ZFE_length.xlsx') # ZFE Daten vom UPTD/ELBE Experiment
# df_ZFE_length = df_ZFE_length.rename(columns={'ELBE_cw':'ELBE$_ref$', 'ELBE_p-like':'ELBE$_{iso}$', 'ELBE_e-like':'ELBE$_{synchro}$','ELBE_FLASH':'ELBE$_{FLASH}$'})
df_ZFE_Draco = pd.read_excel('Dose_length.xlsx', na_filter=True) # ZFE Daten vom Draco Experiment + UPTD Vergleiche
df_melted_ZFE_Draco = df_ZFE_Draco.melt("Dose",var_name="Accelerator",value_name="Length") # pandas Dataframe umstellen
# df_melted_ZFE_Draco_Means =  df_melted_ZFE_Draco.groupby('Accelerator')["Dose"].mean()

df=df_melted_ZFE_Draco
df["category"] = pd.cut(df["Dose"], [0,15,20,25,30], labels=["00-15","15-20","20-25","25-30"]).astype(str)
df_drop= df.dropna()
# df_length = df_drop.groupby(["Accelerator", "category"])["Length"].mean()
df_length = df_drop.groupby(["Accelerator", "category"])["Length"].agg([("Length", "mean"),('stdLen','std')])
df_Dose = df_drop.groupby(["Accelerator", "category"])["Dose"].agg([("Dose", "mean"),('stdDose','std')])
df_DL = pd.concat([df_length, df_Dose], axis = 1)
df_DL.iloc[5,3] = 0.7
df_DL.iloc[9,3] = 0.7

## Daten von Jeanette einlesen & bearbeiten
alldata = pd.read_excel('Zebra_September_2020.xlsx')
alldata['Ox_per_Dose'] = alldata['total_depletion'].astype('float')/alldata['Dose'].astype('float')
alldata_positive = alldata[alldata['Ox_per_Dose']>0] ##wenn depletion negativ ist, heißt dass, dass O2 während der Bestrahlung angestiegen ist (schlechte Abdichtung vermutlich). Diese Daten werden rausgeworfen. 
alldata_oxygen = alldata_positive[alldata_positive['O2_start'] <30] ##hier werden nur kleine O2-Konzentrationen betrachtet. Die Bedingung kann man auch raus nehemen; da ändern sich die Boxen nicht wesentlich. Kommt drauf an, was ihr mit dem Paper aussagen wollt, bzw an welchem O2 Bereich ihr interessiert seid. 
alldata_oxygen = alldata_oxygen.replace(to_replace='DRACO', value='DRACO', regex=True)
alldata_oxygen = alldata_oxygen.replace(to_replace='FLASH', value='UPTD$\\mathrm{_{UHDR}}$') #, regex=True)
alldata_oxygen = alldata_oxygen.replace(to_replace='CW', value='UPTD$\\mathrm{_{ref}}$') #, regex=True)

## Start plots


plot = plt.figure(layout="constrained", figsize=(9,6))
gs0 = GridSpec(2, 10, figure=plot, left=0.1, right=0.9, wspace=0.25, hspace=0.5)
ax2 = plot.add_subplot(gs0[0,:6])
ax3 = plot.add_subplot(gs0[0,6:])

gs1 = GridSpec(2, 10, figure=plot,  left=0.1, right=0.9, wspace=0.25, hspace=0.5)
ax0 = plot.add_subplot(gs1[1,:6])
ax1 = plot.add_subplot(gs1[1,6:])

# plt.subplots_adjust(hspace=0.25)


## ELBE plot
# fig = plt.figure()
# fig, ax = plt.subplots(1, 2, figsize=(12,4))
# Usual boxplot
#ax0 = sns.boxplot(data=df_ZFE_length.iloc[:,0:4], palette=myPal, saturation=1)
sns.boxplot(data=df_ZFE_length.iloc[:,[0,1,2,3,5,6]], palette=myPal, saturation=1, ax=ax0)
ax0.set_xticklabels(['ELBE$\mathrm{_{ref}}$', 'ELBE$\mathrm{_{UHDR}^{iso}}$', 'ELBE$\mathrm{_{UHDR}^{syn}}$',
                       'ELBE$\mathrm{_{UHDR}^{max}}$', 'UPTD$\mathrm{_{ref}}$', 'UPTD$\mathrm{_{UHDR}}$'])
# Add jitter with the swarmplot function
#ax0 = sns.swarmplot(data=df_ZFE_length.iloc[:,0:4], palette=myPal, linewidth=0.9)
sns.swarmplot(data=df_ZFE_length.iloc[:,[0,1,2,3,5,6]], palette=myPal, linewidth=0.8, ax=ax0, s=4)
ax0.set(ylabel='Length normalised to control')
ax0.set(xlabel='Irradiation regime')
# fig.text(0.3, 0.025, 'Accelerator mode', ha='center')
ax0.set_ylim(0.75, 1)
ax0.annotate('Eppendorf tube',
              xy=(4.5, 0.77), 
              va= 'center',
              ha= 'center',
              bbox=dict(boxstyle="round", fc="w")
              )
ax0.text(-1.52,1.005,'c', weight='bold', fontsize=14)

## UPTD slab/eppi plot
#fig2 = plt.figure()
# Usual boxplot
ax1 = sns.boxplot(data=df_ZFE_length.iloc[:,[-1,8,9]], palette=myPal, saturation=1)
# Add jitter with the swarmplot function
# ax2 = sns.stripplot(data=df_ZFE_length.iloc[:,[5,6,8,9]], jitter=0, palette=myPal, linewidth=1)
ax1 = sns.swarmplot(data=df_ZFE_length.iloc[:,[-1,8,9]], palette=myPal, linewidth=1)
plt.ylabel('Length normalised to control')
plt.xticks([1, 2], ['UPTD$\mathrm{_{ref}}$', 'UPTD$\mathrm{_{UHDR}}$'])
ax1.set(xlabel='Irradiation regime')
# fig.text(0.64, 0.025, 'Eppendorf tube', ha='center')
# fig.text(0.81, 0.025, 'Slab', ha='center')
ax1.set_ylim(0.75, 1)
ax1.annotate('Slab',
              xy=(-0.2, 0.77), 
              va= 'center',
              ha= 'center',
              bbox=dict(boxstyle="round", fc="w")
              )
ax1.text(-1.35,1.005,'d', weight='bold', fontsize=14)
# plt.subplots_adjust(wspace=0.25)
# plt.savefig('ZFE_lengths.pdf', bbox_inches='tight')
# plt.savefig('ZFE_lengths.svg')


## Draco/UPTD plot
# fig3, axes = plt.subplots(1, 2, figsize=(12,4))
# ax3 = sns.scatterplot(data=df_melted_ZFE_Draco, x="Dose", y="Length", hue="Accelerator", palette=myPal, alpha=1, linewidth=0, edgecolor='grey')
# sns.scatterplot(data=df_melted_ZFE_Draco, x="Dose", y="Length", hue="Accelerator", palette=myPal, alpha=1, linewidth=0, edgecolor='grey', ax=ax2)
sns.scatterplot(data=df_ZFE_Draco.loc[:,['Dose','Draco']], x="Dose", y="Draco", color=HHGreen, alpha=1, linewidth=0, edgecolor='grey', ax=ax2)
# df_melted_ZFE_Draco.loc[df_melted_ZFE_Draco['Accelerator'] == 'Draco', 'Length'].iloc[2]
# ax2.errorbar(df_DL.loc["Draco","Dose"], df_DL.loc["Draco","Length"], xerr=df_DL.loc["Draco","stdDose"], yerr=df_DL.loc["Draco","stdLen"], fmt='s', color=HHGreen, mec='dimgrey', ecolor='dimgrey',  capsize=5)
ax2.errorbar(df_DL.loc["UPTD_cw","Dose"], df_DL.loc["UPTD_cw","Length"], xerr=df_DL.loc["UPTD_cw","stdDose"], yerr=df_DL.loc["UPTD_cw","stdLen"], fmt='s', color=HHO60, mec=HHGrayDark, ecolor=HHGrayDark, capsize=5)
ax2.errorbar(df_DL.loc["UPTD_FLASH","Dose"], df_DL.loc["UPTD_FLASH","Length"], xerr=df_DL.loc["UPTD_FLASH","stdDose"], yerr=df_DL.loc["UPTD_FLASH","stdLen"], fmt='s', color=HHOrange, mec=HHGrayDark, ecolor=HHGrayDark, capsize=5)
ax2.set(xlabel='Applied Dose [Gy]', ylabel='Length normalised to control')
ax2.set_xlim(4.75, 32)
ax2.set_ylim(0.75, 1.05)
legend_elements = [#Patch(facecolor=HHGreen,alpha=1,label='Draco'),
                   # Patch(facecolor=HHOrange,alpha=1,label='UPTD$_{FLASH}$'),
                   Line2D([0], [0], marker='o', color='w', label='DRACO', markerfacecolor=HHGreen),
                   Line2D([0], [0], marker='s', color='w', label='UPTD$\mathrm{_{UHDR}}$', markerfacecolor=HHOrange),
                   Line2D([0], [0], marker='s', color='w', label='UPTD$\mathrm{_{ref}}$', markerfacecolor=HHO60),
                   # Patch(facecolor=HHO60,alpha=1,label='UPTD$_{FLASH}$')
                   ]
ax2.legend(handles=legend_elements, loc='lower left')
ax2.annotate('Slab',
              xy=(30, 1.03), 
              va= 'center',
              ha= 'center',
              bbox=dict(boxstyle="round", fc="w")
              )
ax2.text(0.0,1.05,'a', weight='bold', fontsize=14)

#ax2.legend(loc="lower left")

## O2 plot Jeanette
# fig4, ax4 = plt.subplots()
sns.boxplot(y='Ox_per_Dose', x='type', 
                 data=alldata_oxygen, color = 'white',
                 fliersize = 0, ax=ax3, order=['DRACO', 'UPTD$\mathrm{_{ref}}$', 'UPTD$\mathrm{_{UHDR}}$'])
# sns.scatterplot(data=alldata_oxygen, x="type", y='Ox_per_Dose', hue="O2_start", palette = pal)
sns.swarmplot(data=alldata_oxygen, x="type", y='Ox_per_Dose', hue="O2_start", palette = pal, linewidth=0, 
              order=['DRACO', 'UPTD$\mathrm{_{ref}}$', 'UPTD$\mathrm{_{UHDR}}$'], ax=ax3)
ax3.legend(title="O$\mathrm{_{2, start}}$ [mmHg]", loc = "upper right", fontsize="7.5", frameon=False)
ax3.set(xlabel='Irradiation regime', ylabel='Depletion per dose [mmHg/Gy]')
ax3.set_ylim(0, 0.15)
ax3.annotate('Slab',
              xy=(-0.2, 0.14), 
              va= 'center',
              ha= 'center',
              bbox=dict(boxstyle="round", fc="w")
              )
ax3.text(-1.35,0.15,'b', weight='bold', fontsize=14)
# plt.subplots_adjust(wspace=0.25)
# plt.savefig('ZFE_Draco.pdf', bbox_inches='tight')
# plt.savefig('ZFE_Draco.svg')

plt.savefig('ZFE_overview.pdf', bbox_inches='tight')
plt.savefig('ZFE_overview.svg')
plt.savefig('ZFE_overview.png')
# show plot
plt.show()